;;; This file is part of turtle, a turtle-graphics system for Scheme.

;;; Copyright (C) 2008  Luca Saiu

;;; This program is free software: you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation, either version 3 of the License, or
;;; (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;; This is the most general version of for, but it's not always
;; comfortable to call. I use this to define three friendlier
;; versions:
(define-macro (for-step variable from to step . body)
  (let ((to-name (gensym))
        (step-name (gensym)))
    `(let ((,variable ,from)
           (,to-name ,to)
           (,step-name ,step))
       (while (<= ,variable ,to-name)
              ,@(append body
                        `((set! ,variable (+ ,variable ,step))))))))

;; Evaluate the body a number of times equal to how-many-times.
;; No iteration variable is visible in the body,
;;
;; Example: (n-times 5 (display "This will be printed five times\n")
(define-macro (n-times how-many-times . body)
  (let ((variable (gensym)))
    `(for ,variable 1 ,how-many-times ,@body)))

;; The classic for loop, with the variable incremented at each iteration.
;;
;; Example: (for i 1 5
;;               (display "i is now ")
;;               (display i)
;;               (newline))
(define-macro (for variable from to . body)
  `(for-step ,variable ,from ,to 1 ,@body))

;; Just like for, but here the variable is *de*cremented at each iteration:
(define-macro (down-for variable from to . body)
  `(for-step ,variable ,from ,to -1 ,@body))
